@@ -26,12 +26,7 @@
 #include <alsa/pcm_external.h>
 #include <alsa/pcm_plugin.h>
 #include AVCODEC_HEADER
-
-#if LIBAVCODEC_VERSION_MAJOR >= 53 && LIBAVCODEC_VERSION_MINOR >= 34
-#include <libavutil/audioconvert.h>
-#include <libavutil/mem.h>
-#define USE_AVCODEC_FRAME
-#endif
+#include <libavutil/avutil.h>
 
 /* some compatibility wrappers */
 #ifndef AV_VERSION_INT
@@ -43,6 +38,12 @@
                                                LIBAVCODEC_VERSION_MICRO)
 #endif
 
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 34, 0)
+#include <libavutil/audioconvert.h>
+#include <libavutil/mem.h>
+#define USE_AVCODEC_FRAME
+#endif
+
 #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(54, 0, 0)
 #ifndef AV_CH_LAYOUT_STEREO
 #define AV_CH_LAYOUT_STEREO	CH_LAYOUT_STEREO
@@ -57,6 +58,9 @@
 #endif
 #endif
 
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(54, 25, 0)
+#define AV_CODEC_ID_AC3 CODEC_ID_AC3
+#endif
 
 struct a52_ctx {
 	snd_pcm_ioplug_t io;
@@ -194,8 +198,8 @@ static int a52_drain(snd_pcm_ioplug_t *io)
 	err = write_out_pending(io, rec);
 	if (err < 0)
 		return err;
-	snd_pcm_drain(rec->slave);
-	return 0;
+
+	return snd_pcm_drain(rec->slave);
 }
 
 /* check whether the areas consist of a continuous interleaved stream */
@@ -478,16 +482,14 @@ static int a52_start(snd_pcm_ioplug_t *io)
 {
 	struct a52_ctx *rec = io->private_data;
 
-	snd_pcm_start(rec->slave);
-	return 0;
+	return snd_pcm_start(rec->slave);
 }
 
 static int a52_stop(snd_pcm_ioplug_t *io)
 {
 	struct a52_ctx *rec = io->private_data;
 
-	snd_pcm_drop(rec->slave);
-	return 0;
+	return snd_pcm_drop(rec->slave);
 }
 
 /* release resources */
@@ -644,10 +646,60 @@ static int a52_close(snd_pcm_ioplug_t *io)
 
 	a52_free(rec);
 	if (rec->slave)
-		snd_pcm_close(rec->slave);
+		return snd_pcm_close(rec->slave);
 	return 0;
 }
 			      
+#if SND_PCM_IOPLUG_VERSION >= 0x102
+static unsigned int chmap4[4] = {
+	SND_CHMAP_FL, SND_CHMAP_FR,
+	SND_CHMAP_RL, SND_CHMAP_RR,
+};
+static unsigned int chmap6[6] = {
+	SND_CHMAP_FL, SND_CHMAP_FR,
+	SND_CHMAP_FC, SND_CHMAP_LFE,
+	SND_CHMAP_RL, SND_CHMAP_RR,
+};
+
+static snd_pcm_chmap_query_t **a52_query_chmaps(snd_pcm_ioplug_t *io ATTRIBUTE_UNUSED)
+{
+	snd_pcm_chmap_query_t **maps;
+	int i;
+
+	maps = calloc(4, sizeof(void *));
+	if (!maps)
+		return NULL;
+	for (i = 0; i < 3; i++) {
+		snd_pcm_chmap_query_t *p;
+		p = maps[i] = calloc((i + 1) * 2 + 2, sizeof(int));
+		if (!p) {
+			snd_pcm_free_chmaps(maps);
+			return NULL;
+		}
+		p->type = SND_CHMAP_TYPE_FIXED;
+		p->map.channels = (i + 1) * 2;
+		memcpy(p->map.pos, i < 2 ? chmap4 : chmap6,
+		       (i + 1) * 2 * sizeof(int));
+	}
+	return maps;
+}
+
+static snd_pcm_chmap_t *a52_get_chmap(snd_pcm_ioplug_t *io)
+{
+	snd_pcm_chmap_t *map;
+
+	if ((io->channels % 2) || io->channels < 2 || io->channels > 6)
+		return NULL;
+	map = malloc((io->channels + 1) * sizeof(int));
+	if (!map)
+		return NULL;
+	map->channels = io->channels;
+	memcpy(map->pos, io->channels < 6 ? chmap4 : chmap6,
+	       io->channels * sizeof(int));
+	return map;
+}
+#endif /* SND_PCM_IOPLUG_VERSION >= 0x102 */
+
 /*
  * callback table
  */
@@ -665,6 +717,10 @@ static snd_pcm_ioplug_callback_t a52_ops = {
 	.poll_descriptors_count = a52_poll_descriptors_count,
 	.poll_descriptors = a52_poll_descriptors,
 	.poll_revents = a52_poll_revents,
+#if SND_PCM_IOPLUG_VERSION >= 0x102
+	.query_chmaps = a52_query_chmaps,
+	.get_chmap = a52_get_chmap,
+#endif /* SND_PCM_IOPLUG_VERSION >= 0x102 */
 };
 
 /*
@@ -863,7 +919,7 @@ SND_PCM_PLUGIN_DEFINE_FUNC(a52)
 	if (rec->codec == NULL)
 		rec->codec = avcodec_find_encoder_by_name("ac3");
 	if (rec->codec == NULL) 
-		rec->codec = avcodec_find_encoder(CODEC_ID_AC3);
+		rec->codec = avcodec_find_encoder(AV_CODEC_ID_AC3);
 	if (rec->codec == NULL) {
 		SNDERR("Cannot find codec engine");
 		err = -EINVAL;
